<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

/*
 * Load Additional Data when getting Order Items by Repository 
 */
class ItemGet
{
    /** @var \FiloBlu\OrderAdditionalData\Model\ItemDataFactory */
    protected $itemDataFactory;

    /** @var \Magento\Sales\Api\Data\ItemExtensionFactory */
    protected $itemExtensionFactory;

    /** @var \Magento\Framework\Serialize\SerializerInterface */
    protected $_serializer;
    
    /** @var \FiloBlu\OrderAdditionalData\Model\AdditionalDataFactory */    
    protected $_additionalDataFactory;    
    /**
     * Init plugin
     *
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \FiloBlu\OrderAdditionalData\Model\ItemDataFactory $itemDataFactory
     * @param \Magento\Sales\Api\Data\OrderExtensionFactory $itemExtensionFactory
     * @param \Magento\Framework\Serialize\SerializerInterface $serializer
     * @param \FiloBlu\OrderAdditionalData\Model\AdditionalDataFactory $additionalDataFactory  
     * @param \FiloBlu\OrderAdditionalData\Model\ProductOptionFactory $productOptionFactory
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context, 
        \FiloBlu\OrderAdditionalData\Model\ItemDataFactory $itemDataFactory,
        \Magento\Sales\Api\Data\OrderItemExtensionFactory $itemExtensionFactory,
        \Magento\Framework\Serialize\SerializerInterface $serializer,
        \FiloBlu\OrderAdditionalData\Model\AdditionalDataFactory $additionalDataFactory,
        \FiloBlu\OrderAdditionalData\Model\ProductOptionFactory $productOptionFactory
    ) {
        $this->itemDataFactory = $itemDataFactory;
        $this->itemExtensionFactory = $itemExtensionFactory;     
        $this->_serializer = $serializer;
        $this->_additionalDataFactory = $additionalDataFactory;
        $this->scopeConfig = $context->getScopeConfig();
        $this->_productOptionFactory=$productOptionFactory;
    }

    /**
     * Get additional data
     *
     * @param \Magento\Sales\Api\OrderItemRepositoryInterface $subject
     * @param \Magento\Sales\Api\Data\OrderItemInterface $resultItem
     * @return \Magento\Sales\Api\Data\OrderItemInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGet(
        \Magento\Sales\Api\OrderItemRepositoryInterface $subject,
        \Magento\Sales\Api\Data\OrderItemInterface $resultItem
    ) {
        $resultItem = $this->getItemAdditionalData($resultItem);
        return $resultItem;
    }

    /**
     * Get additional data for item
     *
     * @param \Magento\Sales\Api\Data\OrderItemInterface $order
     * @return \Magento\Sales\Api\Data\OrderItemInterface
     */
    protected function getItemAdditionalData(\Magento\Sales\Api\Data\OrderItemInterface $item)
    {
        $extensionAttributes = $item->getExtensionAttributes();
        if ($extensionAttributes && $extensionAttributes->getFbOad()) {
            return $item;
        }
        //$additionalData=$this->objectFactory->create();
        $additionalData=array();
        try {
            /** @var \Magento\AdditionalData\Api\Data\MessageInterface $AdditionalData */
            $itemDataObject = $this->itemDataFactory->create()->load($item->getItemId(), "item_id");

            if ($itemDataObject && $itemDataObject->getId() > 0 && $itemDataObject->getAdditionalData()!=null ){
                $additionalData=$this->_serializer->unserialize($itemDataObject->getAdditionalData());
            }
        } catch (NoSuchEntityException $e) {
            return $item;
        }
        /** @var \Magento\Sales\Api\Data\ItemExtension $itemExtension */
        $itemExtension = $extensionAttributes ? $extensionAttributes : $this->itemExtensionFactory->create();
        $item_fields=$this->scopeConfig->getValue('oad/general/item_data_fields', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if($item_fields){
            $item_fields=$this->_serializer->unserialize($item_fields);
            foreach($item_fields as $key=>$item_field){
                if($item_field["object_type"]!="order_item"){
                    unset($item_fields[$key]);
                }
            }
        }
        if($item_fields){
            foreach($item_fields as $item_field){
                $additionalData[$item_field["fieldname"]] = $item->getData($item_field["object_field"]);
            }
        }        
        $additionalDataObject = $this->_additionalDataFactory->create();
        foreach($additionalData as $field => $value){    
            $additionalDataObject->setCustomAttribute($field, $value);
        }
        $itemExtension->setFbOad($additionalDataObject);
        $itemExtension->setData("kpi_full_price", floatval($item->getData("kpi_full_price")));

        $allProductOptions=$item->getProductOptions();
        if (isset($allProductOptions['options']) && !empty($allProductOptions['options'])) {
            $productOptions=$allProductOptions["options"];
            $productOptionsArray=array();
            foreach($productOptions as $key=>$options){
                $productOptionsArray[$key] = $this->_productOptionFactory->create();
                foreach($options as $field=>$value){
                    $productOptionsArray[$key]->setData($field,$value);
                }
            }
            $itemExtension->setProductOptions($productOptionsArray);
        }
        if(array_key_exists("giftcard_created_codes",$allProductOptions)){
            $itemExtension->setData("giftcard_created_codes",$allProductOptions["giftcard_created_codes"]);
        }
        $item->setExtensionAttributes($itemExtension);
        return $item;
    }

    /**
     * @param \Magento\Sales\Api\OrderItemRepositoryInterface $subject
     * @param \Magento\Sales\Model\ResourceModel\Order\Item\Collection $resultOrder
     * @return \Magento\Sales\Model\ResourceModel\Order\Item\Collection
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGetList(
        \Magento\Sales\Api\OrderItemRepositoryInterface $subject,
        \Magento\Sales\Model\ResourceModel\Order\Item\Collection $collection
    ) {
        /** @var  $item */
        foreach ($collection->getItems() as $item) {
            $this->afterGet($subject, $item);
        }
        return $collection;
    }
}
