<?php 
  
namespace FiloBlu\OrderAdditionalData\Setup; 
  
use Magento\Framework\Setup\InstallSchemaInterface; 
use Magento\Framework\Setup\ModuleContextInterface; 
use Magento\Framework\Setup\SchemaSetupInterface; 
use Magento\Framework\DB\Ddl\Table; 
  
class InstallSchema implements InstallSchemaInterface 
{ 
    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context) 
    { 
        $installer = $setup; 
        $installer->startSetup(); 
  
        $tableName = $installer->getTable('sales_oad_order'); 
        if ($installer->getConnection()->isTableExists($tableName) != true) { 
            $table = $installer->getConnection() 
                ->newTable($tableName) 
                ->addColumn( 
                    'data_id', 
                    Table::TYPE_INTEGER, 
                    null, 
                    [ 
                        'identity' => true,
                        'unsigned' => true, 
                        'nullable' => false, 
                        'primary' => true 
                    ], 
                    'Data ID' 
                ) 
                ->addColumn( 
                    'order_id', 
                    Table::TYPE_INTEGER, 
                    0, 
                    [ 
                        'unsigned' => true, 
                        'nullable' => false
                    ], 
                    'Order ID' 
                )                
                ->addColumn( 
                    'additional_data', 
                    Table::TYPE_TEXT,
                    '64k',
                    [
                        'nullable' => true, 
                        'default' => null
                    ], 
                    'Additional Data' 
                )      
                ->addForeignKey(
                    $installer->getFkName(
                        'sales_oad_order',
                        'order_id',
                        'sales_order',
                        'entity_id'
                    ),
                    'order_id',
                    $installer->getTable('sales_order'),
                    'entity_id',
                    Table::ACTION_CASCADE,
                    Table::ACTION_CASCADE
                )                            
                ->setComment('Additional data for order object') 
                ->setOption('type', 'InnoDB') 
                ->setOption('charset', 'utf8'); 
            $installer->getConnection()->createTable($table); 
        } 
  
        $tableName = $installer->getTable('sales_oad_order_item'); 
        if ($installer->getConnection()->isTableExists($tableName) != true) { 
            $table = $installer->getConnection() 
                ->newTable($tableName) 
                ->addColumn( 
                    'data_id', 
                    Table::TYPE_INTEGER, 
                    null, 
                    [ 
                        'identity' => true,
                        'unsigned' => true, 
                        'nullable' => false, 
                        'primary' => true 
                    ], 
                    'Data ID' 
                ) 
                ->addColumn( 
                    'item_id', 
                    Table::TYPE_INTEGER, 
                    0, 
                    [ 
                        'unsigned' => true, 
                        'nullable' => false
                    ], 
                    'Order Item ID' 
                ) 
                ->addColumn( 
                    'additional_data', 
                    Table::TYPE_TEXT, 
                    '64k',
                    ['nullable' => true, 'default' => null], 
                    'Additional Data' 
                )     
                ->addForeignKey(
                    $installer->getFkName(
                        'sales_oad_order_item',
                        'item_id',
                        'sales_order_item',
                        'item_id'
                    ),
                    'item_id',
                    $installer->getTable('sales_order_item'),
                    'item_id',
                    Table::ACTION_CASCADE,
                    Table::ACTION_CASCADE
                )                                    
                ->setComment('Additional data for order item object') 
                ->setOption('type', 'InnoDB') 
                ->setOption('charset', 'utf8'); 
            $installer->getConnection()->createTable($table); 
        } 
        $installer->endSetup(); 
    } 
}